function gameApp()
	app	= {}
	
	' Map width and height
	app.mapWidth		= 640
	app.mapHeight		= 480
	
	app.playField	= [
		0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0,
		0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0,
		0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0,
		0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0,
		0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0,
		0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0,
		0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0,
		0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0,
		0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0
	]
		
	' Functions
	app.Setup			= appSetup
	app.Initialize		= appInitialize
	app.Update			= appUpdate
	app.Render			= appRender
	app.ProcessEvents	= appProcessEvents
	app.AddDefenses		= appAddDefenses
	
	return	app
end function

sub appSetup()
	m.screen	= createScreen()

	if IsHD()
		m.screenRegions	= dfSetupDisplayRegions(m.screen, (940 - m.mapWidth) / 2, 0, m.mapWidth, m.mapHeight)
	else
		m.screenRegions	= dfSetupDisplayRegions(m.screen, (720 - m.mapWidth) / 2, 0, m.mapWidth, m.mapHeight)
	endif

    m.msgport	= CreateObject("roMessagePort")

    m.screen.SetPort(m.msgport)
	
	m.compositor	= CreateObject("roCompositor")
	m.compositor.SetDrawTo(m.screenRegions.Main, 0)

	m.controlEvents	= bslUniversalControlEventCodes()
end sub

sub appInitialize()
	' Starting positions
	m.cursorX	= 288
	m.cursorY	= 224

	m.flyX		= -32
	m.flyY		= 96

	m.horseFlyX	= -32
	m.horseFlyY	= 192
	
	m.antX		= -32
	m.antY		= 288
	
	m.beeX		= -32
	m.beeY		= 352

	' Load graphics xml
	xml			= ReadAsciiFile("pkg:/gfx/gfx.xml")
	m.bitmapset	= dfNewBitmapSet(xml)
		
	if m.bitmapset = invalid
		print "Unable to load gfx.xml or gfx.xml has errors"
	endif
	
	m.bkgBitmap			= CreateObject("roBitmap", "pkg:/gfx/Background.png")
	m.panelBitmap		= CreateObject("roBitmap", "pkg:/gfx/Panel.png")
	m.cursorBitmap		= CreateObject("roBitmap", "pkg:/gfx/Cursor.png")

	m.flySprite			= m.compositor.NewAnimatedSprite(m.flyX, m.flyY, m.bitmapset.animations.Fly)
	m.flySprite.SetZ(1)

	m.horseFlySprite	= m.compositor.NewAnimatedSprite(m.horseFlyX, m.horseFlyY, m.bitmapset.animations.HorseFly)
	m.horseFlySprite.SetZ(1)

	m.AntSprite			= m.compositor.NewAnimatedSprite(m.antX, m.antY, m.bitmapset.animations.Ant)
	m.AntSprite.SetZ(1)

	m.BeeSprite			= m.compositor.NewAnimatedSprite(m.beeX, m.beeY, m.bitmapset.animations.Bee)
	m.BeeSprite.SetZ(1)

	m.addDefenses()
end sub

sub appUpdate()
	m.antX = m.antX + 1
	
	if m.antX > 512
		m.antX	= -32
	endif
	
	m.antSprite.MoveTo(m.antX, m.antY)

	m.flyX = m.flyX + 1
	
	if m.flyX > 512
		m.flyX	= -32
	endif
	
	m.flySprite.MoveTo(m.flyX, m.flyY)

	m.horseFlyX = m.horseFlyX + 2
	
	if m.horseFlyX > 512
		m.horseFlyX	= -32
	endif
	
	m.horseFlySprite.MoveTo(m.horseFlyX, m.horseFlyY)

	m.beeX = m.beeX + 4
	
	if m.beeX > 512
		m.beeX	= -32
	endif
	
	m.beeSprite.MoveTo(m.beeX, m.beeY)
end sub

sub appRender()
	m.screenRegions.Main.SetAlphaEnable(false)
	m.screenRegions.Main.DrawObject(0, 0, m.bkgBitmap)
	m.screenRegions.Main.SetAlphaEnable(true) 
	m.screenRegions.Main.DrawObject(m.cursorX, m.cursorY, m.cursorBitmap)
	m.compositor.DrawAll()
	m.screenRegions.Main.SetAlphaEnable(false) 
	m.screenRegions.Main.DrawObject(512, 0, m.panelBitmap)
	m.screen.SwapBuffers()
end sub

sub appProcessEvents(msg)
	moved	= false

	if msg.GetInt() = m.controlEvents.BUTTON_UP_PRESSED
		m.cursorY	= m.cursorY - 32
		moved		= true
	else if msg.GetInt() = m.controlEvents.BUTTON_DOWN_PRESSED
		m.cursorY	= m.cursorY + 32
		moved		= true
	endif

	if msg.GetInt() = m.controlEvents.BUTTON_LEFT_PRESSED
		m.cursorX	= m.cursorX - 32
		moved		= true
	else if msg.GetInt() = m.controlEvents.BUTTON_RIGHT_PRESSED
		m.cursorX	= m.cursorX + 32
		moved		= true
	endif

	if moved
		if m.cursorX < 0
			m.cursorX	= m.cursorX + 32
		else if m.cursorX > m.mapWidth - 32
			m.cursorX	= m.cursorX - 32
		endif
		
		if m.cursorY < 0
			m.cursorY	= m.cursorY + 32
		else if m.cursorY > m.mapHeight - 32
			m.cursorY	= m.cursorY - 32
		endif
	endif
end sub

sub appAddDefenses()
	x			= 0
	y			= 88
	objectIndex	= 0
	
	for iOuterLoop = 0 to 9
		for iInnerLoop = 0 to 16
			if m.playField[objectIndex] = 1
				m.playField[objectIndex]	= m.compositor.NewAnimatedSprite(x, y, m.bitmapset.animations.Can)
				m.playField[objectIndex].SetZ(0)
			end if
		
			objectIndex = objectIndex + 1
			
			x	= x + 32
		end for
		
		x	= 0
		y	= y + 32
	end for
end sub

function IsHD()
    deviceInfo	= CreateObject("roDeviceInfo")
    
    if deviceInfo.GetDisplayType() = "HDTV"
		return true
    endif
	
	return false
end function

function createScreen()
	' Create a double buffered screen
	if IsHD()
		screen = createobject("roScreen", true, 940, 480)
	else
		screen = createobject("roScreen", true, 720, 480)
	endif
	
	return	screen
end function
